@@ -1,3 +1,4 @@
+from datetime import datetime
 from multiprocessing import Process, Queue
 
 import boto3
@@ -21,27 +22,29 @@ class TagClusterResources:
     """
     This class filter cluster resources by cluster name, and update tags when passing input_tags
     """
-    volume_ids = []
-
+    
+    SHORT_ID = 5
+    
     def __init__(self, cluster_name: str = None, cluster_prefix: str = None, input_tags: dict = None,
                  region: str = 'us-east-2', dry_run: str = 'yes'):
+        self.ec2_operations = EC2Operations(region=region)
+        self.iam_operations = IAMOperations()
+        self.utils = Utils(region=region)
         self.ec2_client = boto3.client('ec2', region_name=region)
         self.elb_client = boto3.client('elb', region_name=region)
         self.elbv2_client = boto3.client('elbv2', region_name=region)
         self.iam_client = boto3.client('iam', region_name=region)
-        self.iam_operations = IAMOperations()
+
         self.s3_client = boto3.client('s3')
         self.cluster_prefix = cluster_prefix
         self.cluster_name = cluster_name
         self.cluster_key = self.__init_cluster_name()
         self.input_tags = input_tags
-        self.__get_details_resource_list = Utils().get_details_resource_list
-        self.__get_username_from_instance_id_and_time = CloudTrailOperations(
-            region_name=region).get_username_by_instance_id_and_time
+        self.cloudtrail = CloudTrailOperations(region_name='us-east-1')
+        self.__get_username_from_instance_id_and_time = CloudTrailOperations(region_name=region).get_username_by_instance_id_and_time
         self.dry_run = dry_run
         self.non_cluster_update = TagNonClusterResources(region=region, dry_run=dry_run, input_tags=input_tags)
-        self.ids = []
-        self.ec2_operations = EC2Operations()
+
 
     def __init_cluster_name(self):
         """
@@ -96,7 +99,7 @@ class TagClusterResources:
                 cluster_name = tag['Key']
                 break
         if not found:
-            value = f'{cluster_name.split("/")[-1]}-{resource_id.split("-")[0]}-{resource_id[-4:]}'
+            value = f'{cluster_name.split("/")[-1]}-{resource_id.split("-")[0]}-{resource_id[-self.SHORT_ID:]}'
             tags.append({'Key': 'Name', 'Value': value})
         return tags
 
@@ -119,6 +122,24 @@ class TagClusterResources:
                                     return [i_tag for i_tag in i_tags if i_tag.get('Key') != cluster_name]
         return []
 
+    def get_date_from_date(self, date_time: datetime):
+        return date_time.strftime('%Y/%m/%d')
+
+    def get_date_from_date_time(self, date_time: datetime):
+        return date_time.strftime('%Y/%m/%d %H:%M:%S')
+
+    def __remove_tags_start_with_aws(self, tags: list):
+        """
+        This method removes tags starting with aws
+        @param tags:
+        @return:
+        """
+        filter_tags = []
+        for tag in tags:
+            if not tag.get('Key').startswith('aws'):
+                filter_tags.append(tag)
+        return filter_tags
+
     def __generate_cluster_resources_list_by_tag(self, resources_list: list, input_resource_id: str, ids=None,
                                                  tags: str = 'Tags'):
         """
@@ -139,12 +160,7 @@ class TagClusterResources:
                         instance_tags = self.__get_cluster_tags_by_instance_cluster(cluster_name=tag.get('Key'))
                         add_tags.extend(instance_tags)
                         add_tags = self.__check_name_in_tags(tags=add_tags, resource_id=resource_id)
-                        if resource.get('CreateTime'):
-                            add_tags = self.__remove_launchTime(add_tags)
-                            add_tags.append({'Key': 'CreateTime', 'Value': str(resource.get('CreateTime'))})
-                        elif resource.get('StartTime'):
-                            add_tags = self.__remove_launchTime(add_tags)
-                            add_tags.append({'Key': 'StartTime', 'Value': str(resource.get('StartTime'))})
+                        add_tags = self.__remove_tags_start_with_aws(add_tags)
                         add_tags = self.__filter_resource_tags_by_add_tags(resource.get(tags), add_tags)
                         if add_tags:
                             if self.cluster_name:
@@ -152,12 +168,12 @@ class TagClusterResources:
                                 if cluster_resource_name == self.cluster_name:
                                     if self.dry_run == "no":
                                         self.ec2_client.create_tags(Resources=[resource_id], Tags=add_tags)
-                                        logger.info(add_tags)
+                                        logger.info(f'{input_resource_id} :: {add_tags}')
                                     result_resources_list.append(resource_id)
                             else:
                                 if self.dry_run == "no":
                                     self.ec2_client.create_tags(Resources=[resource_id], Tags=add_tags)
-                                    logger.info(add_tags)
+                                    logger.info(f'{input_resource_id} :: {add_tags}')
                                 result_resources_list.append(resource_id)
         if ids is not None:
             ids.put(sorted(result_resources_list))
@@ -185,7 +201,7 @@ class TagClusterResources:
             if resource.get('VpcId'):
                 for vpc_id in vpc_data.keys():
                     if resource.get('VpcId') == vpc_id:
-                        all_tags = self.__append_input_tags()
+                        all_tags = []
                         all_tags.extend(vpc_data.get(vpc_id))
                         all_tags = self.__check_name_in_tags(tags=all_tags, resource_id=resource_id)
                         all_tags = self.__filter_resource_tags_by_add_tags(resource.get('Tags'), all_tags)
@@ -235,13 +251,26 @@ class TagClusterResources:
         @return:
         """
         instances_list = []
-        ec2s = self.ec2_client.describe_instances()
-        ec2s_data = ec2s['Reservations']
+        ec2s_data = self.ec2_operations.get_instances()
         for items in ec2s_data:
             if items.get('Instances'):
                 instances_list.append(items['Instances'])
         return instances_list
 
+    def remove_creation_date(self, tags: list):
+        return [tag for tag in tags if tag.get('Key') != 'CreationDate']
+
+    def __check_user_in_username_tags(self, tags: list):
+        """
+        This method check User tag in username tahs
+        @param tags:
+        @return:
+        """
+        for tag in tags:
+            if tag.get('Key') == 'User':
+                return True
+        return False
+
     def update_cluster_tags(self, resources: list, queue):
         """
         This method update the Cluster instance tags and returns the updated tags list ids.
@@ -272,10 +301,29 @@ class TagClusterResources:
                                     start_time=item.get('LaunchTime'), resource_id=instance_id,
                                     resource_type='AWS::EC2::Instance')
                                 if username:
-                                    user_tags = self.iam_operations.get_user_tags(username=username)
-                                    add_tags.extend(user_tags)
-                                    add_tags.append({'Key': 'Email', 'Value': f'{username}@redhat.com'})
-                                add_tags.append({'Key': 'LaunchTime', 'Value': str(item.get('LaunchTime'))})
+                                    if username == 'AutoScaling':
+                                        add_tags.append({'Key': 'User', 'Value': username})
+                                        logger.info(f'Autoscaling instance :: {instance_id}')
+                                    else:
+                                        user_tags = self.iam_operations.get_user_tags(username=username)
+                                        if not self.__check_user_in_username_tags(user_tags):
+                                            try:
+                                                user = self.iam_client.get_user(UserName=username)['User']
+                                                username = self.cloudtrail.get_username_by_instance_id_and_time(
+                                                    start_time=user.get('CreateDate'), resource_id=username,
+                                                    resource_type='AWS::IAM::User')
+                                                user_tags = self.iam_operations.get_user_tags(username=username)
+                                            except:
+                                                add_tags.append({'Key': 'User', 'Value': username})
+                                        if user_tags:
+                                            add_tags.extend(user_tags)
+                                            add_tags.append({'Key': 'Email', 'Value': f'{username}@redhat.com'})
+                                        else:
+                                            add_tags.append({'Key': 'User', 'Value': username})
+                                else:
+                                    add_tags.append({'Key': 'User', 'Value': 'NA'})
+                                add_tags.append({'Key': 'LaunchTime', 'Value': self.get_date_from_date_time(item.get('LaunchTime'))})
+                                add_tags = self.remove_creation_date(add_tags)
                                 add_tags = self.__filter_resource_tags_by_add_tags(tags=item.get('Tags'),
                                                                                    search_tags=add_tags)
                                 if add_tags:
@@ -288,16 +336,15 @@ class TagClusterResources:
                     if self.dry_run == 'no':
                         self.ec2_client.create_tags(Resources=instance_ids,
                                                     Tags=cluster_tags.get(cluster_instance_name))
-                        logger.info(cluster_tags.get(cluster_instance_name))
+                        logger.info(f'Cluster :: {cluster_instance_name} :: InstanceId :: {instance_ids} :: {cluster_tags.get(cluster_instance_name)}')
                     result_instance_list.extend(instance_ids)
             else:
                 if self.dry_run == 'no':
                     self.ec2_client.create_tags(Resources=instance_ids,
                                                 Tags=cluster_tags.get(cluster_instance_name))
-                    logger.info(cluster_tags.get(cluster_instance_name))
+                    logger.info(f'Cluster :: {cluster_instance_name} :: InstanceId :: {instance_ids} :: {cluster_tags.get(cluster_instance_name)}')
                 result_instance_list.extend(instance_ids)
-
-        logger.info(f'cluster_instance count: {len(sorted(result_instance_list))} {sorted(result_instance_list)}')
+        logger.info(f'cluster_instance :: {len(result_instance_list)} :: {result_instance_list}')
         if not self.cluster_key:
             jobs = []
             iam_list = [self.cluster_role, self.cluster_s3_bucket, self.cluster_user]
@@ -329,14 +376,15 @@ class TagClusterResources:
             cluster_process.join()
             non_cluster_process.join()
             return ids.get()
+        else:
+            return []
 
     def cluster_volume(self):
         """
         This method return list of cluster's volume according to cluster tag name
         @return:
         """
-        volumes = self.ec2_client.describe_volumes()
-        volumes_data = volumes['Volumes']
+        volumes_data = self.ec2_operations.get_volumes()
         cluster, non_cluster = self.ec2_operations.scan_cluster_non_cluster_resources(volumes_data)
         ids = Queue()
         cluster_process = Process(target=self.__generate_cluster_resources_list_by_tag,
@@ -353,11 +401,10 @@ class TagClusterResources:
         This method return list of cluster's ami according to cluster tag name
         @return:
         """
-        images = self.ec2_client.describe_images(Owners=['self'])
-        images_data = images['Images']
+        images_data = self.ec2_operations.get_images()
         ids = Queue()
         cluster, non_cluster = self.ec2_operations.scan_cluster_non_cluster_resources(images_data)
-        cluster_process = Process(target=self.__generate_cluster_resources_list_by_tag, args=(cluster, 'ImageId',ids, ))
+        cluster_process = Process(target=self.__generate_cluster_resources_list_by_tag, args=(cluster, 'ImageId', ids, ))
         non_cluster_process = Process(target=self.non_cluster_update.update_ami, args=(non_cluster,))
         cluster_process.start()
         non_cluster_process.start()
@@ -370,8 +417,7 @@ class TagClusterResources:
         This method return list of cluster's snapshot according to cluster tag name
         @return:
         """
-        snapshots = self.ec2_client.describe_snapshots(OwnerIds=['self'])
-        snapshots_data = snapshots['Snapshots']
+        snapshots_data = self.ec2_operations.get_snapshots()
         ids = Queue()
         cluster, non_cluster = self.ec2_operations.scan_cluster_non_cluster_resources(snapshots_data)
         cluster_process = Process(target=self.__generate_cluster_resources_list_by_tag, args=(cluster, 'SnapshotId',ids, ))
@@ -387,8 +433,7 @@ class TagClusterResources:
         This method return security group data
         @return:
         """
-        security_groups = self.ec2_client.describe_security_groups()
-        return security_groups['SecurityGroups']
+        return self.ec2_operations.get_security_groups()
 
     def cluster_security_group(self):
         """
@@ -405,8 +450,7 @@ class TagClusterResources:
         This method return list of cluster's elastic ip according to cluster tag name
         @return:
         """
-        elastic_ips = self.ec2_client.describe_addresses()
-        elastic_ips_data = elastic_ips['Addresses']
+        elastic_ips_data = self.ec2_operations.get_elastic_ips()
         elastic_ips = self.__generate_cluster_resources_list_by_tag(resources_list=elastic_ips_data,
                                                                     input_resource_id='AllocationId')
         logger.info(f'cluster_elastic_ip count: {len(sorted(elastic_ips))} {sorted(elastic_ips)}')
@@ -417,8 +461,7 @@ class TagClusterResources:
         This method return list of cluster's network interface according to cluster tag name
         @return:
         """
-        network_interfaces = self.ec2_client.describe_network_interfaces()
-        network_interfaces_data = network_interfaces['NetworkInterfaces']
+        network_interfaces_data = self.ec2_operations.get_network_interface()
         network_interface_ids = self.__generate_cluster_resources_list_by_tag(resources_list=network_interfaces_data,
                                                                               input_resource_id='NetworkInterfaceId',
                                                                               tags='TagSet')
@@ -432,8 +475,7 @@ class TagClusterResources:
         @return:
         """
         result_resources_list = []
-        load_balancers = self.elb_client.describe_load_balancers()
-        load_balancers_data = load_balancers['LoadBalancerDescriptions']
+        load_balancers_data = self.ec2_operations.get_load_balancers()
         for resource in load_balancers_data:
             resource_id = resource['LoadBalancerName']
             tags = self.elb_client.describe_tags(LoadBalancerNames=[resource_id])
@@ -446,8 +488,7 @@ class TagClusterResources:
                             if not instance_tags:
                                 all_tags = self.__append_input_tags(item.get('Tags'))
                             all_tags.extend(instance_tags)
-                            all_tags = self.__remove_launchTime(all_tags)
-                            all_tags.append({'Key': 'CreatedTime', 'Value': str(resource.get('CreatedTime'))})
+                            all_tags = self.__remove_tags_start_with_aws(all_tags)
                             all_tags = self.__filter_resource_tags_by_add_tags(item.get('Tags'), all_tags)
                             if all_tags:
                                 if self.cluster_name:
@@ -480,8 +521,7 @@ class TagClusterResources:
         @return:
         """
         result_resources_list = []
-        load_balancers = self.elbv2_client.describe_load_balancers()
-        load_balancers_data = load_balancers['LoadBalancers']
+        load_balancers_data = self.ec2_operations.get_load_balancers_v2()
         for resource in load_balancers_data:
             resource_id = resource['LoadBalancerArn']
             tags = self.elbv2_client.describe_tags(ResourceArns=[resource_id])
@@ -494,8 +534,7 @@ class TagClusterResources:
                             if not instance_tags:
                                 all_tags = self.__append_input_tags(item.get('Tags'))
                             all_tags.extend(instance_tags)
-                            all_tags = self.__remove_launchTime(all_tags)
-                            all_tags.append({'Key': 'CreatedTime', 'Value': str(resource.get('CreatedTime'))})
+                            all_tags = self.__remove_tags_start_with_aws(all_tags)
                             all_tags = self.__filter_resource_tags_by_add_tags(item.get('Tags'), all_tags)
                             if all_tags:
                                 if self.cluster_name:
@@ -527,8 +566,7 @@ class TagClusterResources:
         This method return list of cluster's vpc according to cluster tag name
         @return:
         """
-        vpcs = self.ec2_client.describe_vpcs()
-        vpcs_data = vpcs['Vpcs']
+        vpcs_data = self.ec2_operations.get_vpcs()
         vpc_ids = self.__generate_cluster_resources_list_by_tag(resources_list=vpcs_data, input_resource_id='VpcId')
         logger.info(f'cluster_vpc count: {len(sorted(vpc_ids))} {sorted(vpc_ids)}')
         self.cluster_network_acl()
@@ -540,8 +578,7 @@ class TagClusterResources:
         Missing OpenShift Tags for it based on VPCs
         @return:
         """
-        vpcs = self.ec2_client.describe_vpcs()
-        vpcs_data = vpcs['Vpcs']
+        vpcs_data = self.ec2_operations.get_vpcs()
         vpc_ids = {}
         for vpc in vpcs_data:
             if vpc.get('Tags'):
@@ -556,8 +593,7 @@ class TagClusterResources:
         This method return list of cluster's subnet according to cluster tag name
         @return:
         """
-        subnets = self.ec2_client.describe_subnets()
-        subnets_data = subnets['Subnets']
+        subnets_data = self.ec2_operations.get_subnets()
         subnet_ids = self.__generate_cluster_resources_list_by_tag(resources_list=subnets_data,
                                                                    input_resource_id='SubnetId')
         logger.info(f'cluster_subnet count: {len(sorted(subnet_ids))} {sorted(subnet_ids)}')
@@ -568,8 +604,7 @@ class TagClusterResources:
         This method return list of cluster's route table according to cluster tag name
         @return:
         """
-        route_tables = self.ec2_client.describe_route_tables()
-        route_tables_data = route_tables['RouteTables']
+        route_tables_data = self.ec2_operations.get_route_tables()
         route_table_ids = self.__generate_cluster_resources_list_by_tag(resources_list=route_tables_data,
                                                                         input_resource_id='RouteTableId')
         logger.info(f'cluster_route_table count: {len(sorted(route_table_ids))} {sorted(route_table_ids)}')
@@ -580,8 +615,7 @@ class TagClusterResources:
         This method return list of cluster's route table internet gateway according to cluster tag name
         @return:
         """
-        internet_gateways = self.ec2_client.describe_internet_gateways()
-        internet_gateways_data = internet_gateways['InternetGateways']
+        internet_gateways_data = self.ec2_operations.get_internet_gateways()
         internet_gateway_ids = self.__generate_cluster_resources_list_by_tag(resources_list=internet_gateways_data,
                                                                              input_resource_id='InternetGatewayId')
         logger.info(
@@ -593,8 +627,7 @@ class TagClusterResources:
         This method return list of cluster's dhcp option according to cluster tag name
         @return:
         """
-        dhcp_options = self.ec2_client.describe_dhcp_options()
-        dhcp_options_data = dhcp_options['DhcpOptions']
+        dhcp_options_data = self.ec2_operations.get_dhcp_options()
         dhcp_ids = self.__generate_cluster_resources_list_by_tag(resources_list=dhcp_options_data,
                                                                  input_resource_id='DhcpOptionsId')
         logger.info(f'cluster_dhcp_option count: {len(sorted(dhcp_ids))} {sorted(dhcp_ids)}')
@@ -605,8 +638,7 @@ class TagClusterResources:
         This method return list of cluster's vpc endpoint according to cluster tag name
         @return:
         """
-        vpc_endpoints = self.ec2_client.describe_vpc_endpoints()
-        vpc_endpoints_data = vpc_endpoints['VpcEndpoints']
+        vpc_endpoints_data = self.ec2_operations.get_vpce()
         vpc_endpoint_ids = self.__generate_cluster_resources_list_by_tag(resources_list=vpc_endpoints_data,
                                                                          input_resource_id='VpcEndpointId')
         logger.info(f'cluster_vpc_endpoint count: {len(sorted(vpc_endpoint_ids))} {sorted(vpc_endpoint_ids)}')
@@ -617,8 +649,7 @@ class TagClusterResources:
         This method return list of cluster's nat gateway according to cluster tag name
         @return:
         """
-        nat_gateways = self.ec2_client.describe_nat_gateways()
-        nat_gateways_data = nat_gateways['NatGateways']
+        nat_gateways_data = self.ec2_operations.get_nat_gateways()
         nat_gateway_id = self.__generate_cluster_resources_list_by_tag(resources_list=nat_gateways_data,
                                                                        input_resource_id='NatGatewayId')
         logger.info(f'cluster_nat_gateway count: {len(sorted(nat_gateway_id))} {sorted(nat_gateway_id)}')
@@ -630,8 +661,7 @@ class TagClusterResources:
         Missing OpenShift Tags for it based on VPCs
         @return:
         """
-        network_acls = self.ec2_client.describe_network_acls()
-        network_acls_data = network_acls['NetworkAcls']
+        network_acls_data = self.ec2_operations.get_nacls()
         network_acl_ids = self.__generate_cluster_resources_list_by_vpc(resources_list=network_acls_data,
                                                                         input_resource_id='NetworkAclId')
         logger.info(f'cluster_network_acl count: {len(network_acl_ids)}, {network_acl_ids}')
@@ -653,33 +683,37 @@ class TagClusterResources:
                 cluster_key = self.cluster_name if self.cluster_key else cluster_name
                 if cluster_key:
                     # starts with cluster name, search for specific role name for fast scan (a lot of roles)
-                    role_name_list = [f"{cluster_key}-master-role", f"{cluster_key}-worker-role"]
-
-                    for role_name in role_name_list:
-                        try:
-                            role = self.iam_client.get_role(RoleName=role_name)
-                            role_data = role['Role']
-                            all_tags = []
-                            instance_tags = self.__get_cluster_tags_by_instance_cluster(
-                                cluster_name=f'{self.cluster_prefix}{cluster_key}')
-                            if not instance_tags:
-                                all_tags = self.__append_input_tags(role_data.get('Tags'))
-                            else:
-                                all_tags.extend(instance_tags)
-                            all_tags = self.__remove_launchTime(all_tags)
-                            all_tags.append({'Key': 'CreateDate', 'Value': str(role_data.get('CreateDate'))})
-                            all_tags = self.__filter_resource_tags_by_add_tags(role_data.get('Tags'), all_tags)
-                            if all_tags:
-                                if self.dry_run == 'no':
-                                    try:
-                                        all_tags.extend(role_data.get('Tags'))
-                                        self.iam_client.tag_role(RoleName=role_name, Tags=all_tags)
-                                        logger.info(all_tags)
-                                    except Exception as err:
-                                        logger.exception(f'Tags are already updated, {err}')
-                                result_role_list.append(role_data['Arn'])
-                        except Exception as err:
-                            logger.exception(f'Missing cluster role name: {role_name}, {err}')
+                    role_name_list = []
+                    roles = self.iam_operations.get_roles()
+                    for role in roles:
+                        if cluster_key in role.get('RoleName'):
+                            role_name_list.append(role.get('RoleName'))
+                    if role_name_list:
+                        for role_name in role_name_list:
+                            try:
+                                role = self.iam_client.get_role(RoleName=role_name)
+                                role_data = role['Role']
+                                all_tags = []
+                                instance_tags = self.__get_cluster_tags_by_instance_cluster(
+                                    cluster_name=f'{self.cluster_prefix}{cluster_key}')
+                                if not instance_tags:
+                                    all_tags = self.__append_input_tags(role_data.get('Tags'))
+                                else:
+                                    all_tags.extend(instance_tags)
+                                all_tags = self.__remove_tags_start_with_aws(all_tags)
+                                all_tags = self.__filter_resource_tags_by_add_tags(role_data.get('Tags'), all_tags)
+                                if all_tags:
+                                    if self.dry_run == 'no':
+                                        try:
+                                            self.iam_client.tag_role(RoleName=role_name, Tags=all_tags)
+                                            logger.info(all_tags)
+                                        except Exception as err:
+                                            logger.exception(f'Tags are already updated, {err}')
+                                    result_role_list.append(role_data['Arn'])
+                            except Exception as err:
+                                logger.exception(f'Missing cluster role name: {role_name}, {err}')
+                    else:
+                        logger.info(f'Missing role for cluster {cluster_name}')
         logger.info(f'cluster_role count: {len(sorted(result_role_list))} {sorted(result_role_list)}')
         return sorted(result_role_list)
 
@@ -695,8 +729,7 @@ class TagClusterResources:
             cluster_names.append(self.cluster_name)
         if cluster_names:
             for cluster_name in cluster_names:
-                users = self.__get_details_resource_list(self.iam_client.list_users, input_tag='Users',
-                                                         check_tag='Marker')
+                users = self.iam_operations.get_users()
                 # return self.__generate_cluster_resources_list_by_tag(resources_list=users_data,
                 #                                                      input_resource_id='UserId')
                 cluster_name = self.cluster_name if self.cluster_key else cluster_name
@@ -714,12 +747,10 @@ class TagClusterResources:
                                     if not instance_tags:
                                         all_tags = self.__append_input_tags(data.get('Tags'))
                                     all_tags.extend(instance_tags)
-                                    all_tags = self.__remove_launchTime(all_tags)
-                                    all_tags.append({'Key': 'CreateDate', 'Value': str(data.get('CreateDate'))})
+                                    all_tags = self.__remove_tags_start_with_aws(all_tags)
                                     all_tags = self.__filter_resource_tags_by_add_tags(data.get('Tags'), all_tags)
                                     if all_tags:
                                         if self.dry_run == 'no':
-                                            all_tags.extend(data.get('Tags'))
                                             self.iam_client.tag_user(UserName=user_name, Tags=all_tags)
                                             logger.info(all_tags)
                                         result_user_list.append(user_name)
@@ -782,8 +813,7 @@ class TagClusterResources:
                                 if not instance_tags:
                                     add_tags = self.__append_input_tags(bucket_tags)
                                 add_tags.extend(instance_tags)
-                                add_tags = self.__remove_launchTime(add_tags)
-                                add_tags.append({'Key': 'CreationDate', 'Value': str(bucket.get('CreationDate'))})
+                                add_tags = self.__remove_tags_start_with_aws(add_tags)
                                 add_tags = self.__filter_resource_tags_by_add_tags(bucket_tags, add_tags)
                                 if add_tags:
                                     if self.dry_run == 'no':
