@@ -1,4 +1,6 @@
 import boto3
+
+from cloud_governance.common.aws.ec2.ec2_operations import EC2Operations
 from cloud_governance.common.logger.init_logger import logger
 from cloud_governance.common.aws.utils.utils import Utils
 
@@ -31,9 +33,10 @@ class ZombieClusterResources:
         self.delete = delete
         self.cluster_tag = cluster_tag
         self.resource_name = resource_name
-        self.delete_ec2_resource = DeleteEC2Resources(self.ec2_client, self.elb_client, self.elbv2_client)
+        self.delete_ec2_resource = DeleteEC2Resources(self.ec2_client, self.elb_client, self.elbv2_client, region=region)
         self.delete_iam_resource = DeleteIAMResources(iam_client=self.iam_client)
         self.delete_s3_resource = DeleteS3Resources(s3_client=self.s3_client, s3_resource=self.s3_resource)
+        self.ec2_operations = EC2Operations(region=region)
         self.__get_details_resource_list = Utils().get_details_resource_list
 
     def _all_cluster_instance(self):
@@ -46,8 +49,7 @@ class ZombieClusterResources:
         regions_data = self.ec2_client.describe_regions()
         for region in regions_data['Regions']:
             self.__ec2 = boto3.client('ec2', region_name=region['RegionName'])
-            ec2s = self.__ec2.describe_instances()
-            ec2s_data = ec2s['Reservations']
+            ec2s_data = self.__get_details_resource_list(func_name=self.ec2_client.describe_instances, input_tag='Reservations', check_tag='NextToken')
             for items in ec2s_data:
                 if items.get('Instances'):
                     instances_list.append(items['Instances'])
@@ -68,8 +70,7 @@ class ZombieClusterResources:
         """
         instances_list = []
         result_instance = {}
-        ec2s = self.ec2_client.describe_instances()
-        ec2s_data = ec2s['Reservations']
+        ec2s_data = self.ec2_operations.get_instances()
         for items in ec2s_data:
             if items.get('Instances'):
                 instances_list.append(items['Instances'])
@@ -179,14 +180,13 @@ class ZombieClusterResources:
                     zombie_resources[key] = value
         return zombie_resources
 
-    def zombie_cluster_volume(self, vpc_id: str = ''):
+    def zombie_cluster_volume(self, vpc_id: str = '', cluster_tag_vpc: str = ''):
         """
         This method return list of cluster's volume according to cluster tag name and cluster name data
         delete only available resource that related to cluster
         """
         available_volumes = []
-        volumes_data = self.__get_details_resource_list(self.ec2_client.describe_volumes, input_tag='Volumes',
-                                                        check_tag='NextToken')
+        volumes_data = self.ec2_operations.get_volumes()
         # filter in-use resource
         for volume in volumes_data:
             if volume['State'] == 'available':
@@ -199,12 +199,11 @@ class ZombieClusterResources:
                 self.delete_ec2_resource.delete_zombie_resource(resource_id=zombie, resource='ec2_volume')
         return zombies
 
-    def zombie_cluster_ami(self, vpc_id: str = ''):
+    def zombie_cluster_ami(self, vpc_id: str = '', cluster_tag_vpc: str = ''):
         """
         This method return list of cluster's ami according to cluster tag name and cluster name data
         """
-        images = self.ec2_client.describe_images(Owners=['self'])
-        images_data = images['Images']
+        images_data = self.ec2_operations.get_images()
         exist_ami = self.__get_cluster_resources(resources_list=images_data, input_resource_id='ImageId')
         zombies = self.__get_zombie_resources(exist_ami)
         if zombies and self.delete:
@@ -216,12 +215,11 @@ class ZombieClusterResources:
                     logger.exception(f'Cannot deregister_image: {zombie}, {err}')
         return zombies
 
-    def zombie_cluster_snapshot(self, vpc_id: str = ''):
+    def zombie_cluster_snapshot(self, vpc_id: str = '', cluster_tag_vpc: str = ''):
         """
         This method return list of cluster's snapshot according to cluster tag name and cluster name data
         """
-        snapshots = self.ec2_client.describe_snapshots(OwnerIds=['self'])
-        snapshots_data = snapshots['Snapshots']
+        snapshots_data = self.ec2_operations.get_snapshots()
         exist_snapshot = self.__get_cluster_resources(resources_list=snapshots_data,
                                                       input_resource_id='SnapshotId')
         zombies = self.__get_zombie_resources(exist_snapshot)
@@ -230,43 +228,97 @@ class ZombieClusterResources:
                 self.delete_ec2_resource.delete_zombie_resource(resource='ebs_snapshots', resource_id=zombie)
         return zombies
 
-    def zombie_cluster_security_group(self, vpc_id: str = ''):
+    def __get_tag_from_resource_tags(self, tags: list, cluster_tag: str):
+        """
+        This method retunrs the cluster tag
+        @param tags:
+        @param cluster_tag:
+        @return:
+        """
+        if cluster_tag:
+            for tag in tags:
+                if cluster_tag == tag.get('Key'):
+                    return tag.get('Key')
+            return ''
+        return ''
+
+    def __get_vpc_tags(self, vpc_id: str, cluster_tag: str):
+        """
+        This method gets tags of vpc
+        @param vpc_id:
+        @return:
+        """
+        if vpc_id:
+            vpcs = self.ec2_client.describe_vpcs(VpcIds=[vpc_id])['Vpcs']
+            if vpcs:
+                vpc = vpcs[0]
+                if vpc.get('Tags'):
+                    for tag in vpc.get('Tags'):
+                        if cluster_tag in tag.get('Key'):
+                            return tag.get('Key')
+        return ''
+
+    def __get_zombies_by_vpc_id(self, vpc_id: str, resources: list, output_tag: str, cluster_tag: str = '', tags: str = 'Tags', input_tag: str = ''):
+        """
+        This method extracts zombies from vpc_id
+        @param vpc_id:
+        @param resources:
+        @param output_tag:
+        @param cluster_tag:
+        @param tags:
+        @return:
+        """
+        ids = {}
+        for resource in resources:
+            if input_tag:
+                for attachment in resource.get(input_tag):
+                    if attachment.get('VpcId') == vpc_id:
+                        if resource.get(tags):
+                            tag = self.__get_tag_from_resource_tags(resource.get(tags), cluster_tag)
+                            if tag:
+                                ids[resource.get(output_tag)] = tag
+                        else:
+                            tag = self.__get_vpc_tags(vpc_id=vpc_id, cluster_tag=cluster_tag)
+                            if tag:
+                                ids[resource.get(output_tag)] = tag
+            elif resource.get('VpcId') == vpc_id:
+                if resource.get(tags):
+                    tag = self.__get_tag_from_resource_tags(resource.get(tags), cluster_tag)
+                    if tag:
+                        ids[resource.get(output_tag)] = tag
+                else:
+                    tag = self.__get_vpc_tags(vpc_id=vpc_id, cluster_tag=cluster_tag)
+                    if tag:
+                        ids[resource.get(output_tag)] = tag
+        return ids
+
+    def zombie_cluster_security_group(self, vpc_id: str = '', cluster_tag_vpc: str = ''):
         """
         This method return list of zombie cluster's security groups compare to existing instances and cluster name data
         :return: list of zombie cluster's security groups
         """
-        security_groups = self.__get_details_resource_list(func_name=self.ec2_client.describe_security_groups,
-                                                           input_tag='SecurityGroups', check_tag='NextToken')
+        security_groups = self.ec2_operations.get_security_groups()
         exist_security_group = self.__get_cluster_resources(resources_list=security_groups,
                                                             input_resource_id='GroupId')
         zombies = self.__get_zombie_resources(exist_security_group)
         if vpc_id and not zombies:
-            zombies = self.__get_cluster_resources_by_vpc_id(vpc_id=vpc_id,
-                                                             resource_data=security_groups,
-                                                             output_tag='GroupId')
+            zombies = self.__get_zombies_by_vpc_id(vpc_id=vpc_id, resources=security_groups, output_tag='GroupId', cluster_tag=cluster_tag_vpc)
         if zombies and self.delete:
-            for zombie in zombies:
-                security_groups = self.__get_details_resource_list(func_name=self.ec2_client.describe_security_groups,
-                                                                   input_tag='SecurityGroups', check_tag='NextToken')
-                vpc_id = self.__extract_vpc_id_from_resource_data(zombie_id=zombie,
-                                                                  resource_data=security_groups,
-                                                                  input_tag='GroupId')
-                zombie_ids = self.__get_cluster_resources_by_vpc_id(vpc_id=vpc_id,
-                                                                    resource_data=security_groups,
-                                                                    output_tag='GroupId')
+            for zombie, cluster_tag in zombies.items():
+                security_groups = self.ec2_operations.get_security_groups()
+                vpc_id = self.__extract_vpc_id_from_resource_data(zombie_id=zombie, resource_data=security_groups, input_tag='GroupId')
+                zombie_ids = self.__get_zombies_by_vpc_id(vpc_id=vpc_id, resources=security_groups, output_tag='GroupId', cluster_tag=cluster_tag)
                 for zombie_id in zombie_ids:
-                    self.delete_ec2_resource.delete_zombie_resource('security_group', resource_id=zombie_id,
-                                                                    vpc_id=vpc_id)
+                    self.delete_ec2_resource.delete_zombie_resource('security_group', resource_id=zombie_id, vpc_id=vpc_id, cluster_tag=cluster_tag)
         return zombies
 
-    def zombie_cluster_elastic_ip(self, vpc_id: str = ''):
+    def zombie_cluster_elastic_ip(self, vpc_id: str = '', cluster_tag_vpc: str = ''):
         """
         This method return list of zombie cluster's elastic ip according to existing instances and cluster name data
         """
         exist_elastic_ip_association = []
         exist_elastic_ip_allocation = []
-        elastic_ips = self.ec2_client.describe_addresses()
-        elastic_ips_data = elastic_ips['Addresses']
+        elastic_ips_data = self.ec2_operations.get_elastic_ips()
         for elastic_ip in elastic_ips_data:
             if elastic_ip.get('AssociationId'):
                 exist_elastic_ip_association.append(elastic_ip)
@@ -283,49 +335,40 @@ class ZombieClusterResources:
                 self.delete_ec2_resource.delete_zombie_resource(resource='elastic_ip', resource_id=zombie,
                                                                 deletion_type='disassociate')
         if zombies_all and self.delete:
-            for zombie in zombies_all:
-                self.delete_ec2_resource.delete_zombie_resource(resource='elastic_ip', resource_id=zombie)
+            for zombie, cluster_tag in zombies_all.items():
+                self.delete_ec2_resource.delete_zombie_resource(resource='elastic_ip', resource_id=zombie, cluster_tag=cluster_tag)
         zombies = {**zombies_all}
         return zombies
 
-    def zombie_cluster_network_interface(self, vpc_id: str = ''):
+    def zombie_cluster_network_interface(self, vpc_id: str = '', cluster_tag_vpc: str = ''):
         """
         This method return list of zombie cluster's network interface according to existing instances and cluster name data
         """
-        network_interfaces_data = self.__get_details_resource_list(
-            func_name=self.ec2_client.describe_network_interfaces,
-            input_tag='NetworkInterfaces', check_tag='NextToken')
+        network_interfaces_data = self.ec2_operations.get_network_interface()
         exist_network_interface = self.__get_cluster_resources(resources_list=network_interfaces_data,
                                                                input_resource_id='NetworkInterfaceId',
                                                                tags='TagSet')
         zombies = self.__get_zombie_resources(exist_network_interface)
         if not zombies and vpc_id:
-            zombies = self.__get_cluster_resources_by_vpc_id(vpc_id=vpc_id,
-                                                             resource_data=network_interfaces_data,
-                                                             output_tag='NetworkInterfaceId')
+            zombies = self.__get_zombies_by_vpc_id(vpc_id=vpc_id, resources=network_interfaces_data, output_tag='NetworkInterfaceId', tags='TagSet', cluster_tag=cluster_tag_vpc)
         if zombies and self.delete:
-            for zombie in zombies:
-                network_interfaces_data = self.__get_details_resource_list(
-                    func_name=self.ec2_client.describe_network_interfaces,
-                    input_tag='NetworkInterfaces', check_tag='NextToken')
+            for zombie, cluster_tag in zombies.items():
+                network_interfaces_data = self.ec2_operations.get_network_interface()
                 vpc_id = self.__extract_vpc_id_from_resource_data(zombie_id=zombie,
                                                                   resource_data=network_interfaces_data,
                                                                   input_tag='NetworkInterfaceId')
-                zombie_ids = self.__get_cluster_resources_by_vpc_id(vpc_id=vpc_id,
-                                                                    resource_data=network_interfaces_data,
-                                                                    output_tag='NetworkInterfaceId')
+                zombie_ids = self.__get_zombies_by_vpc_id(vpc_id=vpc_id, resources=network_interfaces_data, output_tag='NetworkInterfaceId', cluster_tag=cluster_tag, tags='TagSet')
                 for zombie_id in zombie_ids:
-                    self.delete_ec2_resource.delete_zombie_resource(resource='network_interface', resource_id=zombie_id)
+                    self.delete_ec2_resource.delete_zombie_resource(resource='network_interface', resource_id=zombie_id, cluster_tag=cluster_tag)
         return zombies
 
-    def zombie_cluster_load_balancer(self, vpc_id: str = ''):
+    def zombie_cluster_load_balancer(self, vpc_id: str = '', cluster_tag_vpc: str = ''):
         """
         This method return list of cluster's load balancer according to cluster vpc and cluster name data
         """
 
         exist_load_balancer = {}
-        load_balancers = self.elb_client.describe_load_balancers()
-        load_balancers_data = load_balancers['LoadBalancerDescriptions']
+        load_balancers_data = self.ec2_operations.get_load_balancers()
         for resource in load_balancers_data:
             resource_id = resource['LoadBalancerName']
             tags = self.elb_client.describe_tags(LoadBalancerNames=[resource_id])
@@ -342,17 +385,16 @@ class ZombieClusterResources:
                             break
         zombies = self.__get_zombie_resources(exist_load_balancer)
         if zombies and self.delete:
-            for zombie in zombies:
-                self.delete_ec2_resource.delete_zombie_resource(resource='load_balancer', resource_id=zombie)
+            for zombie, cluster_tag in zombies.items():
+                self.delete_ec2_resource.delete_zombie_resource(resource='load_balancer', resource_id=zombie, cluster_tag=cluster_tag)
         return zombies
 
-    def zombie_cluster_load_balancer_v2(self, vpc_id: str = ''):
+    def zombie_cluster_load_balancer_v2(self, vpc_id: str = '', cluster_tag_vpc: str = ''):
         """
         This method return list of cluster's load balancer according to cluster vpc and cluster name data
         """
         exist_load_balancer = {}
-        load_balancers = self.elbv2_client.describe_load_balancers()
-        load_balancers_data = load_balancers['LoadBalancers']
+        load_balancers_data = self.ec2_operations.get_load_balancers_v2()
         for resource in load_balancers_data:
             resource_id = resource['LoadBalancerArn']
             tags = self.elbv2_client.describe_tags(ResourceArns=[resource_id])
@@ -369,8 +411,8 @@ class ZombieClusterResources:
                             break
         zombies = self.__get_zombie_resources(exist_load_balancer)
         if zombies and self.delete:
-            for zombie in zombies:
-                self.delete_ec2_resource.delete_zombie_resource(resource='load_balancer_v2', resource_id=zombie)
+            for zombie, cluster_tag in zombies.items():
+                self.delete_ec2_resource.delete_zombie_resource(resource='load_balancer_v2', resource_id=zombie, cluster_tag=cluster_tag)
         return zombies
 
     def __get_all_exist_vpcs(self):
@@ -378,8 +420,7 @@ class ZombieClusterResources:
         This method return all exist vpc ids (for supporting Network ACL - missing OpenShift tags)
         :return:
         """
-        vpcs_data = self.__get_details_resource_list(func_name=self.ec2_client.describe_vpcs, input_tag='Vpcs',
-                                                     check_tag='NextToken')
+        vpcs_data = self.ec2_operations.get_vpcs()
         vpcs_list = []
         for resource in vpcs_data:
             vpcs_list.append(resource['VpcId'])
@@ -389,8 +430,7 @@ class ZombieClusterResources:
         """
         This method return list of cluster's vpc according to cluster tag name and cluster name data
         """
-        vpcs_data = self.__get_details_resource_list(func_name=self.ec2_client.describe_vpcs, input_tag='Vpcs',
-                                                     check_tag='NextToken')
+        vpcs_data = self.ec2_operations.get_vpcs()
         exist_vpc = self.__get_cluster_resources(resources_list=vpcs_data,
                                                  input_resource_id='VpcId')
         zombies = self.__get_zombie_resources(exist_vpc)
@@ -401,105 +441,80 @@ class ZombieClusterResources:
                        "ENI": self.zombie_cluster_network_interface, "IGW": self.zombie_cluster_internet_gateway,
                        "SUB": self.zombie_cluster_subnet, "EIP": self.zombie_cluster_elastic_ip}
         if zombies and self.delete:
-            for zombie in zombies:
-                self.delete_ec2_resource.delete_zombie_resource(resource='vpc', resource_id=zombie,
-                                                                pending_resources=delete_dict)
+            for zombie, cluster_tag in zombies.items():
+                self.delete_ec2_resource.delete_zombie_resource(resource='vpc', resource_id=zombie, pending_resources=delete_dict, cluster_tag=cluster_tag)
         return zombies
 
-    def zombie_cluster_subnet(self, vpc_id: str = ''):
+    def zombie_cluster_subnet(self, vpc_id: str = '', cluster_tag_vpc: str = ''):
         """
         This method return list of cluster's subnet according to cluster tag name and cluster name data
         """
-        subnets_data = self.__get_details_resource_list(func_name=self.ec2_client.describe_subnets, input_tag='Subnets',
-                                                        check_tag='NextToken')
+        subnets_data = self.ec2_operations.get_subnets()
         exist_subnet = self.__get_cluster_resources(resources_list=subnets_data,
                                                     input_resource_id='SubnetId')
         zombies = self.__get_zombie_resources(exist_subnet)
         if not zombies and vpc_id:
-            zombies = self.__get_cluster_resources_by_vpc_id(vpc_id=vpc_id,
-                                                             resource_data=subnets_data,
-                                                             output_tag='SubnetId')
+            zombies = self.__get_zombies_by_vpc_id(vpc_id=vpc_id, resources=subnets_data, output_tag='SubnetId', cluster_tag=cluster_tag_vpc)
         if zombies and self.delete:
-            for zombie in zombies:
-                subnets_data = self.__get_details_resource_list(func_name=self.ec2_client.describe_subnets,
-                                                                input_tag='Subnets', check_tag='NextToken')
+            for zombie, cluster_tag in zombies.items():
+                subnets_data = self.ec2_operations.get_subnets()
                 vpc_id = self.__extract_vpc_id_from_resource_data(zombie_id=zombie,
                                                                   resource_data=subnets_data,
                                                                   input_tag='SubnetId')
-                zombie_ids = self.__get_cluster_resources_by_vpc_id(vpc_id=vpc_id,
-                                                                    resource_data=subnets_data,
-                                                                    output_tag='SubnetId')
+                zombie_ids = self.__get_zombies_by_vpc_id(vpc_id=vpc_id,  resources=subnets_data, output_tag='SubnetId', cluster_tag=cluster_tag)
                 for zombie_id in zombie_ids:
-                    self.delete_ec2_resource.delete_zombie_resource(resource='subnet', resource_id=zombie_id)
+                    self.delete_ec2_resource.delete_zombie_resource(resource='subnet', resource_id=zombie_id, cluster_tag=cluster_tag)
         return zombies
 
-    def zombie_cluster_route_table(self, vpc_id: str = ''):
+    def zombie_cluster_route_table(self, vpc_id: str = '', cluster_tag_vpc: str = ''):
         """
         This method return list of cluster's route table according to cluster tag name and cluster name data
         """
-        route_tables_data = self.__get_details_resource_list(func_name=self.ec2_client.describe_route_tables,
-                                                             input_tag='RouteTables', check_tag='NextToken')
+        route_tables_data = self.ec2_operations.get_route_tables()
         exist_route_table = self.__get_cluster_resources(resources_list=route_tables_data,
                                                          input_resource_id='RouteTableId')
         zombies = self.__get_zombie_resources(exist_route_table)
         if not zombies and vpc_id:
-            zombies = self.__get_cluster_resources_by_vpc_id(vpc_id=vpc_id,
-                                                             resource_data=route_tables_data,
-                                                             output_tag='RouteTableId')
+            zombies = self.__get_zombies_by_vpc_id(vpc_id=vpc_id, resources=route_tables_data, output_tag='RouteTableId', cluster_tag=cluster_tag_vpc)
         if zombies and self.delete:
-            for zombie in zombies:
-                route_tables_data = self.__get_details_resource_list(func_name=self.ec2_client.describe_route_tables,
-                                                                     input_tag='RouteTables', check_tag='NextToken')
+            for zombie, cluster_tag in zombies.items():
+                route_tables_data = self.ec2_operations.get_route_tables()
                 vpc_id = self.__extract_vpc_id_from_resource_data(zombie_id=zombie,
                                                                   resource_data=route_tables_data,
                                                                   input_tag='RouteTableId')
-                zombie_ids = self.__get_cluster_resources_by_vpc_id(vpc_id=vpc_id,
-                                                                    resource_data=route_tables_data,
-                                                                    output_tag='RouteTableId')
+                zombie_ids = self.__get_zombies_by_vpc_id(vpc_id=vpc_id, resources=route_tables_data, output_tag='RouteTableId', cluster_tag=cluster_tag)
                 for zombie_id in zombie_ids:
-                    self.delete_ec2_resource.delete_zombie_resource(resource='route_table', resource_id=zombie_id,
-                                                                    vpc_id=vpc_id)
+                    self.delete_ec2_resource.delete_zombie_resource(resource='route_table', resource_id=zombie_id, vpc_id=vpc_id, cluster_tag=cluster_tag)
         return zombies
 
-    def zombie_cluster_internet_gateway(self, vpc_id: str = ''):
+    def zombie_cluster_internet_gateway(self, vpc_id: str = '', cluster_tag_vpc: str = ''):
         """
         This method return list of cluster's route table internet gateway according to cluster tag name and cluster name data
         """
-        internet_gateways_data = self.__get_details_resource_list(func_name=self.ec2_client.describe_internet_gateways,
-                                                                  input_tag='InternetGateways', check_tag='NextToken')
+        internet_gateways_data = self.ec2_operations.get_internet_gateways()
         exist_internet_gateway = self.__get_cluster_resources(resources_list=internet_gateways_data,
                                                               input_resource_id='InternetGatewayId')
 
         zombies = self.__get_zombie_resources(exist_internet_gateway)
         if not zombies and vpc_id:
-            zombies = self.__get_cluster_resources_by_vpc_id(vpc_id=vpc_id,
-                                                             resource_data=internet_gateways_data,
-                                                             output_tag='InternetGatewayId',
-                                                             input_tag='Attachments')
+            zombies = self.__get_zombies_by_vpc_id(vpc_id=vpc_id, resources=internet_gateways_data, output_tag='InternetGatewayId', input_tag='Attachments', cluster_tag=cluster_tag_vpc)
         if zombies and self.delete:
-            for zombie in zombies:
-                internet_gateways_data = self.__get_details_resource_list(
-                    func_name=self.ec2_client.describe_internet_gateways,
-                    input_tag='InternetGateways', check_tag='NextToken')
+            for zombie, cluster_tag in zombies.items():
+                internet_gateways_data = self.ec2_operations.get_internet_gateways()
                 vpc_id = self.__extract_vpc_id_from_resource_data(zombie_id=zombie,
                                                                   resource_data=internet_gateways_data,
                                                                   input_tag='InternetGatewayId',
                                                                   output_tag='Attachments')
-                zombie_ids = self.__get_cluster_resources_by_vpc_id(vpc_id=vpc_id,
-                                                                    resource_data=internet_gateways_data,
-                                                                    output_tag='InternetGatewayId',
-                                                                    input_tag='Attachments')
+                zombie_ids = self.__get_zombies_by_vpc_id(vpc_id=vpc_id, resources=internet_gateways_data, output_tag='InternetGatewayId', cluster_tag=cluster_tag, input_tag='Attachments')
                 for zombie_id in zombie_ids:
-                    self.delete_ec2_resource.delete_zombie_resource(resource='internet_gateway', resource_id=zombie_id,
-                                                                    vpc_id=vpc_id)
+                    self.delete_ec2_resource.delete_zombie_resource(resource='internet_gateway', resource_id=zombie_id, vpc_id=vpc_id, cluster_tag=cluster_tag)
         return zombies
 
-    def zombie_cluster_dhcp_option(self, vpc_id: str = ''):
+    def zombie_cluster_dhcp_option(self, vpc_id: str = '', cluster_tag_vpc: str = ''):
         """
         This method return list of cluster's dhcp option according to cluster tag name and cluster name data
         """
-        dhcp_options_data = self.__get_details_resource_list(func_name=self.ec2_client.describe_dhcp_options,
-                                                             input_tag='DhcpOptions', check_tag='NextToken')
+        dhcp_options_data = self.ec2_operations.get_dhcp_options()
         exist_dhcp_option = self.__get_cluster_resources(resources_list=dhcp_options_data,
                                                          input_resource_id='DhcpOptionsId')
         zombies = self.__get_zombie_resources(exist_dhcp_option)
@@ -509,70 +524,58 @@ class ZombieClusterResources:
                 vpc_id = self.__extract_vpc_id_from_resource_data(zombie_id=zombie, resource_data=vpcs,
                                                                   input_tag='DhcpOptionsId')
                 if vpc_id:
-                    self.delete_ec2_resource.delete_zombie_resource(resource='dhcp_options', resource_id=zombie,
-                                                                    vpc_id=vpc_id)
+                    self.delete_ec2_resource.delete_zombie_resource(resource='dhcp_options', resource_id=zombie, vpc_id=vpc_id)
                 else:
                     self.delete_ec2_resource.delete_zombie_resource(resource='dhcp_options', resource_id=zombie)
         return zombies
 
-    def zombie_cluster_vpc_endpoint(self, vpc_id: str = ''):
+    def zombie_cluster_vpc_endpoint(self, vpc_id: str = '', cluster_tag_vpc: str = ''):
         """
         This method return list of cluster's vpc endpoint according to cluster tag name and cluster name data
         """
-        vpc_endpoints_data = self.__get_details_resource_list(func_name=self.ec2_client.describe_vpc_endpoints,
-                                                              input_tag='VpcEndpoints', check_tag='NextToken')
+        vpc_endpoints_data = self.ec2_operations.get_vpce()
         exist_vpc_endpoint = self.__get_cluster_resources(resources_list=vpc_endpoints_data,
                                                           input_resource_id='VpcEndpointId')
         zombies = self.__get_zombie_resources(exist_vpc_endpoint)
         if not zombies and vpc_id:
-            zombies = self.__get_cluster_resources_by_vpc_id(vpc_id=vpc_id,
-                                                             resource_data=vpc_endpoints_data,
-                                                             output_tag='VpcEndpointId')
+            zombies = self.__get_zombies_by_vpc_id(vpc_id=vpc_id, resources=vpc_endpoints_data, output_tag='VpcEndpointId', cluster_tag=cluster_tag_vpc)
         if zombies and self.delete:
-            for zombie in zombies:
-                vpc_endpoints_data = self.__get_details_resource_list(func_name=self.ec2_client.describe_vpc_endpoints,
-                                                                      input_tag='VpcEndpoints', check_tag='NextToken')
+            for zombie, cluster_tag in zombies.items():
+                vpc_endpoints_data = self.ec2_operations.get_vpce()
                 vpc_id = self.__extract_vpc_id_from_resource_data(zombie_id=zombie,
                                                                   resource_data=vpc_endpoints_data,
                                                                   input_tag='VpcEndpointId')
-                zombie_ids = self.__get_cluster_resources_by_vpc_id(vpc_id=vpc_id,
-                                                                    resource_data=vpc_endpoints_data,
-                                                                    output_tag='VpcEndpointId')
+                zombie_ids = self.__get_zombies_by_vpc_id(vpc_id=vpc_id, resources=vpc_endpoints_data, output_tag='VpcEndpointId', cluster_tag=cluster_tag)
                 for zombie_id in zombie_ids:
-                    self.delete_ec2_resource.delete_zombie_resource(resource='vpc_endpoints', resource_id=zombie_id)
+                    self.delete_ec2_resource.delete_zombie_resource(resource='vpc_endpoints', resource_id=zombie_id, cluster_tag=cluster_tag)
         return zombies
 
-    def zombie_cluster_nat_gateway(self, vpc_id: str = ''):
+    def zombie_cluster_nat_gateway(self, vpc_id: str = '', cluster_tag_vpc: str = ''):
         """
         This method return list of zombie cluster's nat gateway according to cluster tag name and cluster name data
         """
-        nat_gateways_data = self.__get_details_resource_list(func_name=self.ec2_client.describe_nat_gateways,
-                                                             input_tag='NatGateways', check_tag='NextToken')
+        nat_gateways_data = self.ec2_operations.get_nat_gateways()
         exist_nat_gateway = self.__get_cluster_resources(resources_list=nat_gateways_data,
                                                          input_resource_id='NatGatewayId')
         zombies = self.__get_zombie_resources(exist_nat_gateway)
         if not zombies and vpc_id:
-            zombies = self.__get_cluster_resources_by_vpc_id(vpc_id=vpc_id, resource_data=nat_gateways_data,
-                                                             output_tag='NatGatewayId')
+            zombies = self.__get_zombies_by_vpc_id(vpc_id=vpc_id, resources=nat_gateways_data, output_tag='NatGatewayId', cluster_tag=cluster_tag_vpc)
         if zombies and self.delete:
-            for zombie in zombies:
-                nat_gateways_data = self.__get_details_resource_list(func_name=self.ec2_client.describe_nat_gateways,
-                                                                     input_tag='NatGateways', check_tag='NextToken')
+            for zombie, cluster_tag in zombies.items():
+                nat_gateways_data = self.ec2_operations.get_nat_gateways()
                 vpc_id = self.__extract_vpc_id_from_resource_data(zombie, nat_gateways_data,
                                                                   input_tag='NatGatewayId')
-                zombie_ids = self.__get_cluster_resources_by_vpc_id(vpc_id=vpc_id, resource_data=nat_gateways_data,
-                                                                    output_tag='NatGatewayId')
+                zombie_ids = self.__get_zombies_by_vpc_id(vpc_id=vpc_id, resources=nat_gateways_data, output_tag='NatGatewayId', cluster_tag=cluster_tag)
                 for zombie_id in zombie_ids:
-                    self.delete_ec2_resource.delete_zombie_resource(resource='nat_gateways', resource_id=zombie_id)
+                    self.delete_ec2_resource.delete_zombie_resource(resource='nat_gateways', resource_id=zombie_id, cluster_tag=cluster_tag)
         return zombies
 
-    def zombie_cluster_network_acl(self, vpc_id: str = ''):
+    def zombie_cluster_network_acl(self, vpc_id: str = '', cluster_tag_vpc: str = ''):
         """
         This method return list of zombie cluster's network acl according to existing vpc id and cluster name data
         """
         exist_network_acl = {}
-        network_acls_data = self.__get_details_resource_list(func_name=self.ec2_client.describe_network_acls,
-                                                             input_tag='NetworkAcls', check_tag='NextToken')
+        network_acls_data = self.ec2_operations.get_nacls()
         for resource in network_acls_data:
             exist_network_acl[resource['NetworkAclId']] = resource['VpcId']
         zombie_resources = {}
@@ -584,21 +587,17 @@ class ZombieClusterResources:
                     zombie_resources[key] = value
         zombies = zombie_resources
         if not zombies and vpc_id:
-            zombies = [network_acl.get('NetworkAclId') for network_acl in network_acls_data
-                       if network_acl.get('VpcId') == vpc_id]
+            zombies = self.__get_zombies_by_vpc_id(vpc_id=vpc_id, resources=network_acls_data, output_tag='NetworkAclId', cluster_tag=cluster_tag_vpc)
         if zombies and self.delete:
-            for zombie in zombies:
-                network_acls_data = self.__get_details_resource_list(func_name=self.ec2_client.describe_network_acls,
-                                                                     input_tag='NetworkAcls', check_tag='NextToken')
+            for zombie, cluster_tag in zombies.items():
+                network_acls_data = self.ec2_operations.get_nacls()
                 if not vpc_id:
                     vpc_id = self.__extract_vpc_id_from_resource_data(zombie_id=zombie,
                                                                       resource_data=network_acls_data,
                                                                       input_tag='NetworkAclId')
-                zombie_ids = self.__get_cluster_resources_by_vpc_id(vpc_id=vpc_id, resource_data=network_acls_data,
-                                                                    output_tag='NetworkAclId')
+                zombie_ids = self.__get_zombies_by_vpc_id(vpc_id=vpc_id, resources=network_acls_data, output_tag='NetworkAclId', cluster_tag=cluster_tag)
                 for zombie_id in zombie_ids:
-                    self.delete_ec2_resource.delete_zombie_resource(resource='network_acl', resource_id=zombie_id,
-                                                                    vpc_id=vpc_id)
+                    self.delete_ec2_resource.delete_zombie_resource(resource='network_acl', resource_id=zombie_id, vpc_id=vpc_id, cluster_tag=cluster_tag)
         return zombies
 
     def zombie_cluster_role(self):
